package com.yusukeshibata.universal.canvas;

import java.util.*;
import android.util.Log;
import android.app.Activity;
import android.support.annotation.Nullable;
import com.facebook.react.ReactActivity;
import com.facebook.react.bridge.*;

public class UniversalCanvasContext extends ReactContextBaseJavaModule {

	private ReactApplicationContext reactContext;

	public UniversalCanvasContext(ReactApplicationContext reactContext) {
		super(reactContext);
		this.reactContext = reactContext;
	}

	private UniversalCanvasView getView(int contextId) {
		UniversalCanvasModule module = reactContext.getNativeModule(UniversalCanvasModule.class);
		return module.getView(contextId);
	}

	@Override
	public String getName() {
		return getClass().getSimpleName();
	}

	@ReactMethod
	public void update(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.update();
	}

	// Colors, Styles, and Shadows
	@ReactMethod
	public void setFillStyle(int contextId,String fillStyle) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setFillStyle(fillStyle);
	}

	@ReactMethod
	public void setStrokeStyle(int contextId,String strokeStyle) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setStrokeStyle(strokeStyle);
	}

	@ReactMethod
	public void setShadowColor(int contextId,String shadowColor) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setShadowColor(shadowColor);
	}

	@ReactMethod
	public void setShadowBlur(int contextId,String shadowBlur) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setShadowBlur(shadowBlur);
	}

	@ReactMethod
	public void setShadowOffsetX(int contextId,double shadowOffsetX) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setShadowOffsetX(shadowOffsetX);
	}

	@ReactMethod
	public void setShadowOffsetY(int contextId,double shadowOffsetY) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setShadowOffsetY(shadowOffsetY);
	}

	@ReactMethod
	public void createLinearGradient(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.createLinearGradient();
	}

	@ReactMethod
	public void createPattern(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.createPattern();
	}

	@ReactMethod
	public void createRadialGradient(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.createRadialGradient();
	}

	@ReactMethod
	public void addColorStop(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.addColorStop();
	}

	// Line Styles

	@ReactMethod
	public void setLineCap(int contextId,String lineCap) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setLineCap(lineCap);
	}

	@ReactMethod
	public void setLineJoin(int contextId,String lineJoin) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setLineJoin(lineJoin);
	}

	@ReactMethod
	public void setLineWidth(int contextId,double lineWidth) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setLineWidth(lineWidth);
	}

	@ReactMethod
	public void setMiterLimit(int contextId,double miterLimit) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setMiterLimit(miterLimit);
	}

	@ReactMethod
	public void setLineDash(int contextId,ReadableArray lineDash) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		float[] lineDashArr = new float[lineDash.size()];
		for(int i=0;i<lineDash.size();i++) {
			lineDashArr[i] = (float)lineDash.getDouble(i);
		}
		view.setLineDash(lineDashArr);
	}

	@ReactMethod
	public void setLineDashOffset(int contextId,double lineDashOffset) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setLineDashOffset(lineDashOffset);
	}

	// Rectangle

	@ReactMethod
	public void rect(int contextId,double x,double y,double width,double height) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.rect(x,y,width,height);
	}

	@ReactMethod
	public void fillRect(int contextId,double x,double y,double width,double height) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.fillRect(x,y,width,height);
	}

	@ReactMethod
	public void strokeRect(int contextId,double x,double y,double width,double height) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.strokeRect(x,y,width,height);
	}

	@ReactMethod
	public void clearRect(int contextId,double x,double y,double width,double height) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.clearRect(x,y,width,height);
	}

	// Paths

	@ReactMethod
	public void fill(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.fill();
	}

	@ReactMethod
	public void stroke(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.stroke();
	}

	@ReactMethod
	public void beginPath(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.beginPath();
	}

	@ReactMethod
	public void moveTo(int contextId,double x,double y) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.moveTo(x,y);
	}

	@ReactMethod
	public void closePath(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.closePath();
	}

	@ReactMethod
	public void lineTo(int contextId,double x,double y) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.lineTo(x,y);
	}

	@ReactMethod
	public void clip(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.clip();
	}

	@ReactMethod
	public void quadraticCurveTo(int contextId,double x,double y,double cx,double cy) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.quadraticCurveTo(x,y,cx,cy);
	}

	@ReactMethod
	public void bezierCurveTo(int contextId,double x,double y,double c1x,double c1y,double c2x,double c2y) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.bezierCurveTo(x,y,c1x,c1y,c2x,c2y);
	}

	@ReactMethod
	public void arc(int contextId,double x,double y,double r,double sAngle,double eAngle,Boolean counterclockwise) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.arc(x,y,r,sAngle,eAngle,counterclockwise);
	}

	@ReactMethod
	public void arcTo(int contextId,double x1,double y1,double x2,double y2,double r) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.arcTo(x1,y1,x2,y2,r);
	}

	//@ReactMethod
	//public void isPointInPath(int contextId,double x,double y) {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	//	view.isPointInPath(x,y);
	//}

	// Transformations

	@ReactMethod
	public void scale(int contextId,double scalewidth,double scaleheight) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.scale(scalewidth,scaleheight);
	}

	@ReactMethod
	public void rotate(int contextId,double angle) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.rotate(angle);
	}

	@ReactMethod
	public void translate(int contextId,double x,double y) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.translate(x,y);
	}

	@ReactMethod
	public void transform(int contextId,double a,double b,double c, double d,double e,double f) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.transform(a,b,c,d,e,f);
	}

	@ReactMethod
	public void setTransform(int contextId,double a,double b,double c, double d,double e,double f) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setTransform(a,b,c,d,e,f);
	}

	// Text

	@ReactMethod
	public void setFont(int contextId,String font) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setFont(font);
	}

	@ReactMethod
	public void setTextAlign(int contextId,String textAlign) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setTextAlign(textAlign);
	}

	@ReactMethod
	public void setTextBaseline(int contextId,String textBaseline) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setTextBaseline(textBaseline);
	}

	@ReactMethod
	public void fillText(int contextId,String font,double x,double y,ReadableArray args) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		double maxWidth = args!=null && args.size() > 0 ? args.getDouble(0) : 0.0;
		view.fillText(font,x,y,maxWidth);
	}

	@ReactMethod
	public void strokeText(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.strokeText();
	}

	// @ReactMethod
	// public void measureText(ReadableArray text) {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.measureText(text.getString(0));
	// }

	// // Image Drawing

	// @ReactMethod
	// public void drawImage(Image image,ReadableArray x,ReadableArray y) {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.drawImage(image,x,y);
	// }

	// Pixel Manipulation

	// @ReactMethod
	// public void setData() {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.setData();
	// }

	// @ReactMethod
	// public void createImageData() {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.createImageData();
	// }

	// @ReactMethod
	// public void getImageData() {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.getImageData();
	// }

	// @ReactMethod
	// public void putImageData() {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.putImageData();
	// }

	// Compositing

	@ReactMethod
	public void setGlobalAlpha(int contextId,double alpha) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setGlobalAlpha(alpha);
	}

	@ReactMethod
	public void setGlobalCompositeOperation(int contextId,String globalCompositeOperation) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.setGlobalCompositeOperation(globalCompositeOperation);
	}

	// Other

	@ReactMethod
	public void save(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.save();
	}

	@ReactMethod
	public void restore(int contextId) {
		UniversalCanvasView view = this.getView(contextId);
		if(view == null) return;
		view.restore();
	}

	// @ReactMethod
	// public void createEvent(int contextId) {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.createEvent();
	// }

	// @ReactMethod
	// public void getContext(int contextId) {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// }

	// @ReactMethod
	// public void toDataURL(int contextId) {
	//	UniversalCanvasView view = this.getView(contextId);
	//	if(view == null) return;
	// 	view.toDataURL();
	// }
}

