package com.yusukeshibata.universal.canvas;

import java.util.Vector;
import android.util.Log;
import android.util.DisplayMetrics;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Point;
import android.graphics.Path;
import android.graphics.Matrix;
import android.graphics.Bitmap;
import android.graphics.DashPathEffect;
import android.graphics.Rect;
import android.graphics.Color;
import android.view.View;
import android.view.WindowManager;
import android.view.MotionEvent;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactContext;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.uimanager.events.RCTEventEmitter;

public class UniversalCanvasView extends View {

	protected Paint currentPaint = null;
	protected Vector<Paint> paints = new Vector<Paint>();
	protected Paint paint = new Paint(Paint.ANTI_ALIAS_FLAG);
	protected int fillColor = 0;
	protected int strokeColor = 0;
	protected Path path = new Path();
	protected Canvas offCanvas = new Canvas();
	protected Bitmap currentBitmap = null;
	protected Bitmap drawingBitmap = null;
	protected Vector<Bitmap> bitmaps = new Vector<Bitmap>();
	protected float lineDashOffset = 0;
	protected float[] lineDash;
	protected Matrix mat = new Matrix();
	protected int width = 0;
	protected int height = 0;
	protected float density;

	public UniversalCanvasView(Context context) {
		super(context);
		save();
	}

	@Override
	public void onSizeChanged(int w,int h,int oldw,int oldh) {
		DisplayMetrics metrics = new DisplayMetrics();
		((WindowManager) getContext().getSystemService(Context.WINDOW_SERVICE))
			.getDefaultDisplay().getMetrics(metrics);
		this.width = w;
		this.height = h;
		this.density = metrics.density;
		//mat.setScale(metrics.density,metrics.density);
		Log.v("ReactNative","width:"+w+",height:"+h);
	}

	private void shiftBitmap() {
		if(bitmaps.size() == 0) return;
		Bitmap firstBitmap = bitmaps.firstElement();
		bitmaps.removeElement(firstBitmap);
		drawingBitmap = firstBitmap;
	}

	private void pushBitmap() {
		if(currentBitmap != null) {
			bitmaps.add(currentBitmap);
		}
		Bitmap newBitmap = Bitmap.createBitmap(
			this.width,
			this.height,
			Bitmap.Config.ARGB_8888
		);
		currentBitmap = newBitmap;
		offCanvas.setBitmap(newBitmap);
	}

	public void update() {
		shiftBitmap();
		if(drawingBitmap != null) {
			postInvalidate();
		}
		pushBitmap();
	}

	// Colors, Styles, and Shadows
	public void setFillStyle(String fillStyle) {
		int r = Integer.parseInt(fillStyle.substring(1,3),16);
		int g = Integer.parseInt(fillStyle.substring(3,5),16);
		int b = Integer.parseInt(fillStyle.substring(5,7),16);
		this.fillColor = Color.argb(0xff,r,g,b);
	}
	public void setStrokeStyle(String strokeStyle) {
		int r = Integer.parseInt(strokeStyle.substring(1,3),16);
		int g = Integer.parseInt(strokeStyle.substring(3,5),16);
		int b = Integer.parseInt(strokeStyle.substring(5,7),16);
		this.strokeColor = Color.argb(0xff,r,g,b);
	}
	public void setShadowColor(String shadowColor) {
		//TODO
	}
	public void setShadowBlur(String shadowBlur) {
		//TODO
	}
	public void setShadowOffsetX(double shadowOffsetX) {
		//TODO
	}
	public void setShadowOffsetY(double shadowOffsetY) {
		//TODO
	}
	public void createLinearGradient() {
		//TODO
	}
	public void createPattern() {
		//TODO
	}
	public void createRadialGradient() {
		//TODO
	}
	public void addColorStop() {
		//TODO
	}

	// Line Styles
	public void setLineCap(String lineCap) {
		currentPaint.setStrokeCap(Paint.Cap.valueOf(lineCap.toUpperCase()));
	}
	public void setLineJoin(String lineJoin) {
		currentPaint.setStrokeJoin(Paint.Join.valueOf(lineJoin.toUpperCase()));
	}
	public void setLineWidth(double lineWidth) {
		currentPaint.setStrokeWidth((float)lineWidth);
	}
	public void setMiterLimit(double miterLimit) {
		currentPaint.setStrokeMiter((float)miterLimit);
	}
	public void setLineDash(float[] lineDash) {
		this.lineDash = lineDash;
		currentPaint.setPathEffect(new DashPathEffect(this.lineDash, this.lineDashOffset));
	}
	public void setLineDashOffset(double lineDashOffset) {
		this.lineDashOffset = (float)lineDashOffset;
		currentPaint.setPathEffect(new DashPathEffect(this.lineDash, this.lineDashOffset));
	}

	// Rectangle
	public void rect(double x,double y, double width, double height) {
		path.addRect((float)x,(float)y,(float)(x+width),(float)(y+height),Path.Direction.CW);
	}
	public void fillRect(double x,double y, double width, double height) {
		currentPaint.setStyle(Paint.Style.FILL);
		currentPaint.setColor(fillColor);
		offCanvas.scale(density,density);
		offCanvas.drawRect((float)x,(float)y,(float)width,(float)height,currentPaint);
		offCanvas.scale(1/density,1/density);
	}
	public void strokeRect(double x,double y, double width, double height) {
		currentPaint.setStyle(Paint.Style.STROKE);
		currentPaint.setColor(strokeColor);
		offCanvas.scale(density,density);
		offCanvas.drawRect((float)x,(float)y,(float)width,(float)height,currentPaint);
		offCanvas.scale(1/density,1/density);
	}
	public void clearRect(double x,double y, double width, double height) {
		//TODO
	}

	// Paths
	public void fill() {
		currentPaint.setStyle(Paint.Style.FILL);
		currentPaint.setColor(fillColor);
		offCanvas.scale(density,density);
		offCanvas.drawPath(path,currentPaint);
		offCanvas.scale(1/density,1/density);
		path.reset();
	}
	public void stroke() {
		currentPaint.setStyle(Paint.Style.STROKE);
		currentPaint.setColor(strokeColor);
		offCanvas.scale(density,density);
		offCanvas.drawPath(path,currentPaint);
		offCanvas.scale(1/density,1/density);
		path.reset();
	}
	public void beginPath() {
		path.reset();
	}
	public void moveTo(double x,double y) {
		path.moveTo((float)x,(float)y);
	}
	public void closePath() {
		path.close();
	}
	public void lineTo(double x,double y) {
		path.lineTo((float)x,(float)y);
	}
	public void clip() {
		offCanvas.clipPath(path);
		path.reset();
	}
	public void quadraticCurveTo(double x, double y, double cx1, double cy1) {
		path.quadTo((float)x,(float)y,(float)cx1,(float)cy1);
	}
	public void bezierCurveTo(double x, double y, double c1x, double c1y, double c2x, double c2y) {
		path.cubicTo((float)x,(float)y,(float)c1x,(float)c1y,(float)c2x,(float)c2y);
	}
	public void arc(double x,double y,double r,double sAngle,double eAngle,Boolean counterclockwise) {
		path.addArc((float)(x-r),(float)(y-r),(float)(x+r),(float)(y+r),(float)sAngle,(float)eAngle);
	}
	public void arcTo(double x1,double y1,double x2,double y2,double r) {
		//TODO
	}
	public Boolean isPointInPath(double x,double y) {
		//TODO
		return false;
	}

	// Transformations
	public void scale(double scalewidth,double scaleheight) {
		offCanvas.scale((float)scalewidth,(float)scaleheight);
	}
	public void rotate(double angle) {
		offCanvas.rotate((float)angle);
	}
	public void translate(double x,double y) {
		offCanvas.translate((float)x,(float)y);
	}
	public void transform(double a,double b,double c,double d,double e,double f) {
		Matrix mat = new Matrix();
		mat.setValues(new float[]{(float)a,(float)b,(float)c,(float)d,(float)e,(float)f});
		offCanvas.concat(mat);
	}
	public void setTransform(double a,double b,double c,double d,double e,double f) {
		Matrix mat = new Matrix();
		mat.setValues(new float[]{(float)a,(float)b,(float)c,(float)d,(float)e,(float)f});
		offCanvas.setMatrix(mat);
	}

	// Text
	public void setFont(String font) {
		//TODO
	}
	public void setTextAlign(String textAlign) {
		//TODO
	}
	public void setTextBaseline(String textBaseline) {
		//TODO
	}
	public void fillText(String text,double x,double y,double maxWidth) {
		//TODO
	}
	public void strokeText() {
		//TODO
	}
	public double measureText(String text) {
		//TODO
		return 0.0f;
	}

	// Image Drawing
	public void drawImage() {
		//TODO
	}

	// Pixel Manipulation
	public void setData() {
		//TODO
	}
	public void createImageData() {
		//TODO
	}
	public void getImageData() {
		//TODO
	}
	public void putImageData() {
		//TODO
	}

	// Compositing
	public void setGlobalAlpha(double alpha) {
		//TODO
	}
	public void setGlobalCompositeOperation(String globalCompositeOperation) {
		//TODO
	}

	// Other
	public void save() {
		paints.add(currentPaint);
		currentPaint = new Paint(Paint.ANTI_ALIAS_FLAG);
	}
	public void restore() {
		if(paints.size() == 0) return;
		Paint lastPaint = paints.lastElement();
		paints.removeElement(lastPaint);
		currentPaint = lastPaint;
	}
	public void createEvent() {
		//TODO
	}
	public void toDataURL() {
		//TODO
	}

	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		if(drawingBitmap != null) {
			canvas.drawBitmap(drawingBitmap,mat,paint);
		}
	}
}
