import React from 'react'
import { NativeModules, View, requireNativeComponent } from 'react-native'
import Context from './context'

const { UniversalCanvasModule, UniversalCanvasContext } = NativeModules

class UniversalCanvas extends React.Component {
  constructor(props) {
    super(props)
    this.onTouchStart = this.onTouchStart.bind(this)
    this.onTouchMove = this.onTouchMove.bind(this)
    this.onTouchEnd = this.onTouchEnd.bind(this)
    this._context = new Context()
    this.state = {}
  }
  componentDidMount() {
    let that = this
    UniversalCanvasModule.createContext(context => {
      that._context.setContext(context)
      that.setState({
        context: context
      })
    })
  }
  get width() {
    return this.props.width
  }
  get height() {
    return this.props.height
  }
  update() {
    this._context.update()
  }
  getContext() {
    return this._context
  }
  onTouchStart(evt) {
    let { onTouchStart } = this.props
    let { pageX, pageY } = evt.nativeEvent
    if(onTouchStart) onTouchStart({
      clientX: pageX,
      clientY: pageY
    })
  }
  onTouchMove(evt) {
    let { onTouchMove } = this.props
    let { pageX, pageY } = evt.nativeEvent
    if(onTouchMove) onTouchMove({
      clientX: pageX,
      clientY: pageY
    })
  }
  onTouchEnd(evt) {
    let { onTouchEnd } = this.props
    if(onTouchEnd) onTouchEnd({})
  }
  render() {
    let { width, height} = this.props
    let { context } = this.state
    return (
      <NativeCanvas
        onTouchStart={this.onTouchStart}
        onTouchMove={this.onTouchMove}
        onTouchEnd={this.onTouchEnd}
        context={context}
        style={{
          width: width,
          height: height
        }}
      />
    )
  }
}
UniversalCanvas.propTypes = {
  ...View.propTypes,
  context: React.PropTypes.number
}
UniversalCanvas.defaultProps = {
}

const NativeCanvas = requireNativeComponent('UniversalCanvas', UniversalCanvas, {
  nativeOnly: {
    onLayout: true
  }
})

module.exports = UniversalCanvas
